# =========================================================================
# THIS FILE IS AUTOMATICALLY GENERATED BY MINILLA.
# DO NOT EDIT DIRECTLY.
# =========================================================================

use 5.008_001;
use strict;

use Module::Build::Tiny 0.035;

die "OS unsupported\n" if $^O eq "MSWin32";
die "OS unsupported\n" if $^O eq "amigaos";
die "OS unsupported\n" if $^O eq "os2";
die "OS unsupported\n" if $^O eq "symbian";
die "OS unsupported\n" if $^O eq "dos";
die "OS unsupported\n" if $^O eq "VMS";
die "OS unsupported\n" if $^O eq "VOS";
die "OS unsupported\n" if $^O eq "riscos";
die "OS unsupported\n" if $^O eq "MacOS";
die "OS unsupported\n" if $^O eq "vmesa";

Build_PL();

